@@ -1,4 +1,4 @@
-// Copyright 2019 Bryant Luk
+// Copyright 2022 Bryant Luk
 //
 // Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
 // http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
@@ -6,28 +6,40 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-//! # LsRules
-//!
-//! [Little Snitch][little_snitch] is like an application based firewall.  It allows
-//! a user to control what connections can be made to and from local applications.
-//! You can choose to allow or deny connections based on a set of rules (such as the
-//! domain or hostname being connected to, the ports used, the application receiving
-//! or making the connection, etc.).
-//!
-//! [LsRules][lsrules] is a file format which specifies rules which Little Snitch
-//! can use. This library is a [Serde][serde] model for serializing and
-//! deserializing `.lsrules` files.
-//!
-//! [little_snitch]: https://www.obdev.at/products/littlesnitch/index.html
-//! [lsrules]: https://help.obdev.at/littlesnitch/ref-lsrules-file-format
-//! [serde]: https://serde.rs
+#![doc = include_str!("../README.md")]
+#![cfg_attr(not(feature = "std"), no_std)]
+#![cfg_attr(docsrs, feature(doc_cfg))]
+#![warn(
+    missing_copy_implementations,
+    missing_debug_implementations,
+    rust_2018_idioms,
+    unused_lifetimes,
+    unused_qualifications
+)]
+
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+extern crate alloc;
+
+use core::{fmt, str::FromStr};
+
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+use alloc::{
+    format,
+    string::{String, ToString},
+    vec::Vec,
+};
+#[cfg(feature = "std")]
+use std::{
+    format,
+    string::{String, ToString},
+    vec::Vec,
+};
 
 use serde::{de::Visitor, Deserialize, Deserializer, Serialize, Serializer};
-use std::fmt;
-use std::str::FromStr;
+use serde_derive::{Deserialize, Serialize};
 
 /// The container for all data.
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[derive(Clone, Debug, Default, PartialEq, Serialize, Deserialize)]
 #[non_exhaustive]
 pub struct LsRules {
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -58,20 +70,6 @@ pub struct LsRules {
     pub denied_remote_notes: Option<String>,
 }
 
-impl Default for LsRules {
-    fn default() -> Self {
-        LsRules {
-            name: None,
-            description: None,
-            rules: None,
-            denied_remote_domains: None,
-            denied_remote_hosts: None,
-            denied_remote_addresses: None,
-            denied_remote_notes: None,
-        }
-    }
-}
-
 /// A specific rule.
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[non_exhaustive]
@@ -155,7 +153,7 @@ struct RemoteVisitor;
 impl<'de> Visitor<'de> for RemoteVisitor {
     type Value = Remote;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
         formatter.write_str("a string value")
     }
 
@@ -217,7 +215,7 @@ struct DirectionVisitor;
 impl<'de> Visitor<'de> for DirectionVisitor {
     type Value = Direction;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
         formatter.write_str("a string value")
     }
 
@@ -276,7 +274,7 @@ struct ActionVisitor;
 impl<'de> Visitor<'de> for ActionVisitor {
     type Value = Action;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
         formatter.write_str("a string value")
     }
 
@@ -334,7 +332,7 @@ struct PriorityVisitor;
 impl<'de> Visitor<'de> for PriorityVisitor {
     type Value = Priority;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
         formatter.write_str("a string value")
     }
 
@@ -395,7 +393,7 @@ struct PortsVisitor;
 impl<'de> Visitor<'de> for PortsVisitor {
     type Value = Ports;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
         formatter.write_str("a string, integer, or range value")
     }
 
@@ -403,25 +401,24 @@ impl<'de> Visitor<'de> for PortsVisitor {
     where
         E: serde::de::Error,
     {
-        match &v.to_lowercase()[..] {
-            "any" => Ok(Ports::Any),
-            _ => {
-                if let Ok(v) = u16::from_str(v) {
-                    return Ok(Ports::Single(v));
-                }
+        if matches!(&v.to_lowercase()[..], "any") {
+            return Ok(Ports::Any);
+        }
 
-                let ports: Vec<&str> = v.split("-").collect();
-                if ports.len() == 2 {
-                    if let Ok(p1) = u16::from_str(ports[0]) {
-                        if let Ok(p2) = u16::from_str(ports[1]) {
-                            return Ok(Ports::Range(p1, p2));
-                        }
-                    }
-                }
+        if let Ok(v) = u16::from_str(v) {
+            return Ok(Ports::Single(v));
+        }
 
-                Ok(Ports::Unknown(String::from(v)))
+        let ports: Vec<&str> = v.split('-').collect();
+        if ports.len() == 2 {
+            if let Ok(p1) = u16::from_str(ports[0]) {
+                if let Ok(p2) = u16::from_str(ports[1]) {
+                    return Ok(Ports::Range(p1, p2));
+                }
             }
         }
+
+        Ok(Ports::Unknown(String::from(v)))
     }
 }
 
@@ -438,8 +435,11 @@ impl<'de> Deserialize<'de> for Ports {
 mod tests {
     use super::*;
 
+    #[cfg(all(feature = "alloc", not(feature = "std")))]
+    use alloc::vec;
+
     #[test]
-    fn test_default_rules() -> Result<(), serde_json::Error> {
+    fn test_default_rules() {
         let rules: LsRules = LsRules::default();
         assert_eq!(rules.name, None);
         assert_eq!(rules.description, None);
@@ -448,7 +448,6 @@ mod tests {
         assert_eq!(rules.denied_remote_hosts, None);
         assert_eq!(rules.denied_remote_addresses, None);
         assert_eq!(rules.denied_remote_notes, None);
-        Ok(())
     }
 
     #[test]
@@ -457,7 +456,7 @@ mod tests {
 {
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         assert_eq!(rules.name, None);
         assert_eq!(rules.description, None);
         Ok(())
@@ -471,7 +470,7 @@ mod tests {
     "description": "Blocks access to popular social media sites."
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         assert_eq!(rules.name, Some(String::from("Social Media Block List")));
         assert_eq!(
             rules.description,
@@ -489,7 +488,7 @@ mod tests {
     "denied-remote-domains": ["facebook.com", "twitter.com", "youtube.com"]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         assert_eq!(rules.name, Some(String::from("Social Media Block List")));
         assert_eq!(
             rules.description,
@@ -521,7 +520,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         assert_eq!(rules.name, Some(String::from("LaunchBar Software Update")));
         assert_eq!(
             rules.description,
@@ -560,7 +559,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         assert_eq!(rules.name, Some(String::from("LaunchBar Software Update")));
         assert_eq!(
             rules.description,
@@ -600,7 +599,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
@@ -626,7 +625,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
@@ -652,7 +651,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
@@ -682,7 +681,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
@@ -704,7 +703,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
@@ -726,7 +725,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
@@ -748,7 +747,7 @@ mod tests {
   ]
 }
 "#;
-        let rules: LsRules = serde_json::from_str(&json)?;
+        let rules: LsRules = serde_json::from_str(json)?;
         let rules = rules.rules.expect("expecting rules");
         assert_eq!(rules.len(), 1);
         let rule = rules.first().expect("first rule to exist");
